﻿/*
AwesomeNET by xeetsh - Maximilian T. Schmitt

Licensed under the Unity Asset Store Terms of Service and EULA. See https://unity3d.com/legal/as_terms for a complete version of the license

AwesomeNET Doccumentation: https://docs.awesomenet.xeet.sh
Contact/Support/Feature requests: https://xeet.sh/contact.php
Twitter: @xeetsh
*/

using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEditor;
using System.Reflection;
using System;

[CustomEditor(typeof(NetworkGameObjectConnector))]
public class NetworkGameObjectConnectorEditor : Editor {

    int m_scriptIndex = 0;
    int m_methodIndex = 0;
    string id = "";
    public override void OnInspectorGUI() {
        // Font Styles
        GUIStyle styleBold = new GUIStyle();
        styleBold.fontStyle = FontStyle.Bold;

        NetworkGameObjectConnector ngoc = (NetworkGameObjectConnector) target;

        EditorGUI.BeginChangeCheck();

        // Object ID
        EditorGUILayout.LabelField("Object ID: " + ngoc.m_objectID);
        if (GUILayout.Button("Renew ID"))
            ngoc.RenewID();
            
        if (ngoc.m_objectID == "")
            ngoc.RenewID();

        EditorGUILayout.Space();

        EditorGUILayout.LabelField("New Network Action", styleBold);

        // Get a list of all attached scripts
        MonoBehaviour[] scripts;
        scripts = ngoc.gameObject.GetComponents<MonoBehaviour>();
        string[] scriptNames = new string[scripts.Length];
        for(int i = 0; i < scripts.Length; i++) {
            scriptNames[i] = scripts[i].GetType().ToString();
        }
        m_scriptIndex = EditorGUILayout.Popup("Script", m_scriptIndex, scriptNames);

        // Get a list of all methods in the selected scripts
        MethodInfo[] methods = scripts[m_scriptIndex].GetType().GetMethods(BindingFlags.Instance | BindingFlags.Public);
        string[] methodNames = new string[methods.Length];
        for(int i = 0; i < methods.Length; i++) {
            methodNames[i] = methods[i].Name;
        }
        m_methodIndex = EditorGUILayout.Popup("Function", m_methodIndex, methodNames);

        // Get the type of the argument of the method
        ParameterInfo[] parameters = methods[m_methodIndex].GetParameters();
        string[] parameterTypes = new string[parameters.Length];
        string parameterNames = " ";
        for (int i = 0; i < parameters.Length; i++) {
            parameterTypes[i] = parameters[i].ParameterType.AssemblyQualifiedName;
            parameterNames += parameters[i].ParameterType.Name;
            parameterNames += " ";
        }

        // ID box
        id = EditorGUILayout.TextField("ID (empty for random)", id);

        // Add button
        if (GUILayout.Button("Add")) {
            if (id == "")
                ngoc.m_commands.Add(new NetworkCommand(scripts[m_scriptIndex], methodNames[m_methodIndex], parameterTypes, parameterNames));
            else
                ngoc.m_commands.Add(new NetworkCommand(scripts[m_scriptIndex], methodNames[m_methodIndex], parameterTypes, parameterNames, id));
        }

        // Populate the command list
        if (ngoc.m_commands.Count != 0) {
            EditorGUILayout.Space();
            EditorGUILayout.LabelField("Network Actions", styleBold);
        }
        
        NetworkCommand removeMe = null;
        foreach (NetworkCommand nc in ngoc.m_commands) {
            EditorGUILayout.LabelField(nc.component.GetType().ToString() + "." + nc.command + " (" + nc.arguments + ")");
            EditorGUILayout.LabelField("Action ID: " + nc.actionID);

            // Remove button
            if (GUILayout.Button("-")) {
                removeMe = nc;
            }
            EditorGUILayout.Space();
        }
        if (removeMe != null)
            ngoc.m_commands.Remove(removeMe);

        
        if (EditorGUI.EndChangeCheck())
            serializedObject.ApplyModifiedProperties();
        
    }
}
