﻿/*
AwesomeNET by xeetsh - Maximilian T. Schmitt

Licensed under the Unity Asset Store Terms of Service and EULA. See https://unity3d.com/legal/as_terms for a complete version of the license

AwesomeNET Doccumentation: https://docs.awesomenet.xeet.sh
Contact/Support/Feature requests: https://xeet.sh/contact.php
Twitter: @xeetsh
*/

using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class CubeController : MonoBehaviour {

    void Update() {
        // Left mouse button: Drop cube
        if (Input.GetMouseButtonDown(0)) {
            // Detect if clicked on a cube
            Ray ray = Camera.main.ScreenPointToRay(Input.mousePosition);
            RaycastHit hit;
         
            if (Physics.Raycast(ray, out hit, 100) && hit.transform.gameObject == gameObject) {
                // Local
                // DropCube();

                // -- Network --
                GetComponent<NetworkGameObjectConnector>().ExecuteNetworkCommand("CubeController.DropCube", new object[] {});
            }
        }

        // Right mouse button: launche cube in random direction
        if (Input.GetMouseButtonDown(1)) {
            // Detect if clicked on a cube
            Ray ray = Camera.main.ScreenPointToRay(Input.mousePosition);
            RaycastHit hit;
         
            if (Physics.Raycast(ray, out hit, 100) && hit.transform.gameObject == gameObject) {
                // Local
                // LaunchCube(new Vector3(Random.value, Random.value, Random.value), new Vector3(Random.value, Random.value, Random.value));

                // -- Network --
                GetComponent<NetworkGameObjectConnector>().ExecuteNetworkCommand("CubeController.LaunchCube", new object[] {new Vector3(Random.value, Random.value, Random.value), new Vector3(Random.value, Random.value, Random.value)});
            }
        }
    }

    // Let's the cube drop by enabling it to be affected by gravity
    public void DropCube() {
        GetComponent<Rigidbody>().useGravity = true;
    }

    // Launches the cube by giving it some soft of force and torque
    public void LaunchCube(Vector3 force, Vector3 torque) {
        // Assign some force and touque to the cube
        GetComponent<Rigidbody>().AddForce(force * 10f);
        GetComponent<Rigidbody>().AddTorque(torque * 10f);
    }
}