﻿/*
AwesomeNET by xeetsh - Maximilian T. Schmitt

Licensed under the Unity Asset Store Terms of Service and EULA. See https://unity3d.com/legal/as_terms for a complete version of the license

AwesomeNET Doccumentation: https://docs.awesomenet.xeet.sh
Contact/Support/Feature requests: https://xeet.sh/contact.php
Twitter: @xeetsh
*/

using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System;

[System.Serializable]
public class NetworkGameObjectConnector : MonoBehaviour {
    public static Dictionary<string, NetworkGameObjectConnector> m_connectors = new Dictionary<string, NetworkGameObjectConnector>();
    public static Dictionary<string, string> m_humanRedableIDs = new Dictionary<string, string>();
    NetworkController m_nc;

    [SerializeField]
    public List<NetworkCommand> m_commands = new List<NetworkCommand>();

    [SerializeField]
    public string m_objectID = "";

    void Start() {
        m_nc = GameObject.FindGameObjectWithTag("NetworkController").GetComponent<NetworkController>();

        // Assign ID if object has none
        if (m_objectID == "") {
            RenewID();
        }
        // Add id to list and check if it is an dublicate
        if (!m_connectors.ContainsKey(m_objectID)) {
            m_connectors.Add(m_objectID, this);
        } else {
            throw new NetworkGameObjectConnectorDoubleIDException(String.Format("An connector with the ID: {0} is allready bound to another object: {1}", m_objectID));
        }
    }
    
    public void ExecuteNetworkCommand(string action, object[] argument) {
        string command = "";
        foreach(NetworkCommand nc in m_commands) {
            if (nc.action == action)
                command = nc.actionID;
        }

        if (command != "")
            m_nc.SendGSCommand(m_objectID, command, argument);
        else
            throw new NetworkGameObjectConnectorUnknownNetworkAction(String.Format("The action {0} is not registerd in {1} ObjectID: {2}", action, gameObject.name, m_objectID));
    }

    public void RenewID() {
        m_objectID = System.Guid.NewGuid().ToString();
    }


}

[Serializable]
public class NetworkCommand {
    // Component that contains the method
    public Component component;
    // Name of the method
    public string command;
    // UUID of the action
    public string actionID;
    // Classname.Methodname
    public string action;
    // Public keys of method arguments
    public string[] argumentType;
    // Nicely formated arguments eg: string, int[], Cake[]
    public string arguments;
    public NetworkCommand(Component component, string command, string[] argumentType, string arguments) {
        this.component = component;
        this.command = command;
        this.actionID = System.Guid.NewGuid().ToString();
        this.action = component.GetType().ToString() + "." + command;
        this.argumentType = argumentType;
        this.arguments = arguments;
    }

    public NetworkCommand(Component component, string command, string[] argumentType, string arguments, string id) {
        this.component = component;
        this.command = command;
        this.actionID = id;
        this.action = component.GetType().ToString() + "." + command;
        this.argumentType = argumentType;
        this.arguments = arguments;
    }
}

public class NetworkGameObjectConnectorDoubleIDException : Exception {
    public NetworkGameObjectConnectorDoubleIDException(string message) :base(message) {

    }
}

public class NetworkGameObjectConnectorUnknownNetworkAction : Exception {
    public NetworkGameObjectConnectorUnknownNetworkAction(string message) :base(message) {

    }
}

