﻿/*
AwesomeNET by xeetsh - Maximilian T. Schmitt

Licensed under the Unity Asset Store Terms of Service and EULA. See https://unity3d.com/legal/as_terms for a complete version of the license

AwesomeNET Doccumentation: https://docs.awesomenet.xeet.sh
Contact/Support/Feature requests: https://xeet.sh/contact.php
Twitter: @xeetsh
*/
 
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;
using UnityEngine.UI;

public class MenuController : MonoBehaviour {
    [Header("Connections")]
    public NetworkController m_NetworkController;

    [Header("Matchmaking")]
    public GameObject m_MatchmakingFrame;
    public InputField m_InputFieldMatchmakingHost;
    public InputField m_InputFieldMatchmakingPort;
    public Button m_ButtonMatchmakingConnect;
    public Button m_ButtonOfflineGame;

    [Header("Games")]
    public GameObject m_GameFrame;
    public InputField m_InputFieldGamesName;
    public Button m_ButtonGamesNewOffline;
    public Button m_ButtonGamesNewOnline;
    public Button m_ButtonGamesJoinOnline;
    public Button m_ButtonGamesRefreshGames;
    public Dropdown m_DropdownGamesGames;


    private bool m_connected = false;
    private string[,] m_gameList;

    void Start() {
        // Binding the buttons to the click action
        m_ButtonMatchmakingConnect.onClick.AddListener(() => ButtonOnClick(1));
        m_ButtonOfflineGame.onClick.AddListener(() => ButtonOnClick(2));
        m_ButtonGamesNewOffline.onClick.AddListener(() => ButtonOnClick(3));
        m_ButtonGamesNewOnline.onClick.AddListener(() => ButtonOnClick(4));
        m_ButtonGamesRefreshGames.onClick.AddListener(() => ButtonOnClick(5));
        m_ButtonGamesJoinOnline.onClick.AddListener(() => ButtonOnClick(6));
    }

    void ButtonOnClick(int buttonNumber) {
        switch (buttonNumber){
            // Connect to matchmaking
            case 1:
                // Connect
                if (!m_connected && m_NetworkController.ConnectMatchmaking(m_InputFieldMatchmakingHost.text, int.Parse(m_InputFieldMatchmakingPort.text))) {
                    m_connected = true;
                    m_ButtonGamesNewOnline.interactable = true;
                    m_ButtonGamesRefreshGames.interactable = true;

                    // Search for advertisements
                    ButtonOnClick(5);

                    // Show the games window
                    m_MatchmakingFrame.SetActive(false);
                    m_GameFrame.SetActive(true);
                }
            break;

            // Offline game button
            case 2:
                m_connected = false;
                
                // Show the games window
                m_MatchmakingFrame.SetActive(false);
                m_GameFrame.SetActive(true);
            break;

            // New game offline
            case 3:
                if (m_NetworkController.NewGameOffline("DemoMap", m_InputFieldGamesName.text))
                    SceneManager.LoadScene("DemoMap");
            break;

            // New game online
            case 4:
                if (m_NetworkController.NewGameOnline("DemoMap", m_InputFieldGamesName.text))
                    SceneManager.LoadScene("DemoMap");
            break;

            // Refresh game list
            case 5:
                // Get a list of currently running games
                string[] games = m_NetworkController.GetGameList();

                // Nicely write them into the dropdown menu
                // The games unique id is saved but not shown to the user and is to join the game (case 6)
                if (games != null && games[0] != "0") {
                    List<Dropdown.OptionData> elements = new List<Dropdown.OptionData>();
                    m_gameList = new string[games.Length - 1, 2];
                    for (int i = 1; i < games.Length; i++) {
                        string[] gameData = games[i].Split(';');
                        m_gameList[i - 1, 0] = gameData[0];
                        m_gameList[i - 1, 1] = gameData[2];
                        elements.Add(new Dropdown.OptionData(gameData[1] + " on " + gameData[2]));
                    }
                    m_DropdownGamesGames.options = elements;
                    m_DropdownGamesGames.interactable = true;
                    m_ButtonGamesJoinOnline.interactable = true;
                } else {
                    m_DropdownGamesGames.interactable = false;
                    m_ButtonGamesJoinOnline.interactable = false;
                }     
            break;

            // Join selected game
            case 6:
                if (m_NetworkController.JoinGame(m_gameList[m_DropdownGamesGames.value, 0], m_InputFieldGamesName.text))
                    SceneManager.LoadScene(m_gameList[m_DropdownGamesGames.value, 1]);
            break;

        }
    }
}
